//---------------------------------------------------------------------------
#include <vcl.h>
#pragma hdrstop

#include "TMainForm.h"
//---------------------------------------------------------------------------
#pragma link "TDx_Sound"
#pragma link "TDx_SoundBuffer"
#pragma link "TDSCaps"
#pragma link "TDSBufferDesc"
#pragma link "TDx_Sound_Library_Install"
#pragma resource "*.dfm"
TMainForm *MainForm;
//---------------------------------------------------------------------------
#define Msg(A) 		StatusMemo->Lines->Add(A)
//---------------------------------------------------------------------------
__fastcall TMainForm::TMainForm(TComponent* Owner)
    : TForm(Owner)
{
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::TestButtonClick(TObject *Sender)
{
Msg( "Test button was clicked, playing an example .WAV file that does not exist..." );

// create a default DirectSound device
if (Dx_Sound1->Create(NULL))
	{
	// create a new DirectSoundBuffer and force an error by attempting to load a non-existing .WAV file into it
    Attempted_Retry = false;
	Dx_SoundBuffer1->CreateFromFile("dummy.wav", DSBufferDesc1, Dx_Sound1);
    if (Dx_SoundBuffer1->Created)
	    {
        Dx_SoundBuffer1->Play(0);

		// do something while the sound is playing
		while (Dx_SoundBuffer1->Playing) ; // actually, do nothing

		// destroy the soundbuffer
		Dx_SoundBuffer1->Destroy();

		// destroy the sound device
		Dx_Sound1->Destroy();
	    }
    }
else
    // this is one way of handling errors when Dx_Sound1::Create() fails.
    switch (Dx_Sound1->ErrorValue)
        {
        case DSERR_ALLOCATED:		Msg("After> Dx_Sound1::Create(), Error==DSERR_ALLOCATED"); break;
        case DSERR_INVALIDPARAM:	Msg("After> Dx_Sound1::Create(), Error==DSERR_INVALIDPARAM"); break;
        case DSERR_NOAGGREGATION:	Msg("After> Dx_Sound1::Create(), Error==DSERR_NOAGGREGATION"); break;
        case DSERR_NODRIVER:		Msg("After> Dx_Sound1::Create(), Error==DSERR_NODRIVER"); break;
        case DSERR_OUTOFMEMORY:		Msg("After> Dx_Sound1::Create(), Error==DSERR_OUTOFMEMORY"); break;
        case TDX_ERROR:				Msg("After> Dx_Sound1::Create(), Error==TDX_ERROR"); break;
        case TDX_BADPARAMS:			Msg("After> Dx_Sound1::Create(), Error==TDX_BADPARAMS"); break;
        case TDX_NOTCREATED:		Msg("After> Dx_Sound1::Create(), Error==TDX_NOTCREATED"); break;
        case TDX_ALREADYCREATED:	Msg("After> Dx_Sound1::Create(), Error==TDX_BADPARAMS"); break;
        default:
            Msg( "An error occurred after calling Dx_Sound1::Create()..." );
            Msg( "... the error code: "+TDx_Sound_Library_ErrorString(Dx_Sound1->ErrorValue) );
            Msg( "... the error message: "+TDx_Sound_Library_ErrorMessage(Dx_Sound1->ErrorValue) );
        }

// display a message
Msg( "...finished playing the example .WAV file" );
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::Dx_SoundBuffer1Error(TObject *Sender,
	AnsiString TheFunction, AnsiString TheError, AnsiString TheErrorMessage)
{
Msg( "An error occurred in: "+TheFunction+"..." );
Msg( "... the error code: "+TheError );
Msg( "... the error message: "+TheErrorMessage );

// This is another way of handling errors.
switch (Dx_SoundBuffer1->ErrorValue)
	{
    case DSERR_BUFFERLOST:
    	break; // Do nothing more, OnBufferLost() below will handle it

    case TDX_ERROR:
    	// if the error message originated in Dx_SoundBuffer1::CreateFromFile()
    	if (TheFunction.Pos("::CreateFromFile()")!=0)
        	// try recreating the buffer using a filename that actually exists,
	        // but only do it once in case the CreateFromFile() fails and triggers this event again.
	    	if (!Attempted_Retry)
	        	{
		    	Msg("In> Dx_SoundBuffer1::OnError(), recreating buffer using a .WAV file that exists.");
	        	Dx_SoundBuffer1->CreateFromFile( "..\\..\\Wav\\Begin.wav", DSBufferDesc1, Dx_Sound1 );
	            Attempted_Retry = true;
	            }
        break;
    }
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::Dx_SoundBuffer1BufferLost(TObject *Sender)
{
// this event is called when Dx_SoundBuffer1->ErrorValue == DSERR_BUFFERLOST
// try recreating the buffer using a filename that actually exists
Msg("In> Dx_SoundBuffer1::OnBufferLost(), Restoring lost sound buffer.");
if (Dx_SoundBuffer1->CreateFromFile( "..\\..\\Wav\\Begin.wav", DSBufferDesc1, Dx_Sound1 ))
	Dx_SoundBuffer1->Play(0);
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::QuitButtonClick(TObject *Sender)
{
Close();
}
//---------------------------------------------------------------------------
