//---------------------------------------------------------------------------
#include <vcl.h>
#pragma hdrstop

#include "TMainForm.h"
//---------------------------------------------------------------------------
#pragma link "TDx_Sound"
#pragma link "TDx_SoundBuffer"
#pragma link "TDSCaps"
#pragma link "TDSBufferDesc"
#pragma link "TDx_Sound_Library_Install"
#pragma resource "*.dfm"
//---------------------------------------------------------------------------
TMainForm *MainForm;
//---------------------------------------------------------------------------
__fastcall TMainForm::TMainForm(TComponent* Owner)
    : TForm(Owner)
{
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::FormCreate(TObject *Sender)
{
KnownDevices = new TStringList();
KnownModules = new TStringList();
KnownGuids = new TList();

DirectoryListBox1->Directory = ExtractFilePath(Application->ExeName)+"..\\..\\Wav";
DevicesListBox->Items->Clear();
if (Dx_Sound1->DSEnumerate( NULL ))
    {
    if (KnownDevices->Count>0)
    	{
	    DevicesListBox->ItemIndex = 0;
	    Dx_Sound1->Create( (GUID*) KnownGuids->Items[DevicesListBox->ItemIndex] );
        }
    }
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::FormDestroy(TObject *Sender)
{
if (Dx_Sound1->Created) Dx_Sound1->Destroy();

if (KnownGuids) delete KnownGuids;
if (KnownModules) delete KnownModules;
if (KnownDevices) delete KnownDevices;
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::DevicesListBoxClick(TObject *Sender)
{
if (Dx_Sound1->Created) Dx_Sound1->Destroy();
NotCreatedLabel->Visible = !Dx_Sound1->Create( (GUID*) KnownGuids->Items[DevicesListBox->ItemIndex] );
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::Dx_Sound1Destroy(TObject *Sender)
{
if (Dx_SoundBuffer1->Created) Dx_SoundBuffer1->Destroy();
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::Dx_SoundBuffer1BufferLost(TObject *Sender)
{
Dx_SoundBuffer1->CreateFromFile( Edit1->Text, DSBufferDesc1, Dx_Sound1 );
Dx_SoundBuffer1->Play( 0 );
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::Dx_Sound1DSEnumCallback(TObject *Sender, GUID *TheGUID, AnsiString TheDriverDescription, AnsiString TheDriverModule, void *TheContext, bool &Finished)
{
if (KnownDevices && KnownModules && KnownGuids)
	{
    KnownDevices->Add( TheDriverDescription );
    KnownModules->Add( TheDriverModule );
    KnownGuids->Add( (void*) TheGUID );
    DevicesListBox->Items->Add( TheDriverDescription + ", ("+TheDriverModule+")" );
    }
else
	Finished = true;
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::PlayButtonClick(TObject *Sender)
{
if (Paused)
	{
    Paused = false;
	if (LoopingCheckBox->Checked) Dx_SoundBuffer1->Play(DSBPLAY_LOOPING);
    else Dx_SoundBuffer1->Play(0);
    return;
    }

if (Dx_SoundBuffer1->Created) Dx_SoundBuffer1->Destroy();
if (Edit1->Text.Pos("*.WAV"))
	{
//    StatusMemo->Lines->Add( "Cannot select *.WAV" );
    return;
    }
if (Dx_SoundBuffer1->CreateFromFile( Edit1->Text, DSBufferDesc1, Dx_Sound1 ))
	{
    BufferSizeLabel->Caption = "Buffer Size: "+IntToStr(DSBufferDesc1->BufferBytes)+" bytes";
	if (LoopingCheckBox->Checked) Dx_SoundBuffer1->Play(DSBPLAY_LOOPING);
    else Dx_SoundBuffer1->Play(0);
    }
else
    ShowMessage("Couldn't Create From File");
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::Timer1Timer(TObject *Sender)
{
if (!Dx_SoundBuffer1->Created) return;

if (Dx_SoundBuffer1->Playing)
	PlayingLabel->Caption = "Buffer is Playing...";
else
	PlayingLabel->Caption = "Buffer is Stopped.";

Dx_SoundBuffer1->GetCurrentPosition( &CurrentPlayPos, &CurrentWritePos );
BufferPosLabel->Caption = "Buffer Position: "+IntToStr(CurrentPlayPos);
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::Dx_SoundBuffer1Create(TObject *Sender)
{
if (Dx_SoundBuffer1->Created)
	{
    Dx_SoundBuffer1->GetPan( &Pan );
    Dx_SoundBuffer1->GetVolume( &Volume );
    Dx_SoundBuffer1->GetFrequency( &Frequency );

    PanScrollBar->Position = Pan;
    VolumeScrollBar->Position = Volume;
    FrequencyScrollBar->Position = Frequency/10;	// see bugfix below

    PanLabel->Caption = "Pan: "+IntToStr((int) Pan);
    VolumeLabel->Caption = "Volume: "+IntToStr((int) Volume);
    FrequencyLabel->Caption = "Frequency: "+IntToStr((int) Frequency);
    }
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::PanScrollBarChange(TObject *Sender)
{
Pan = PanScrollBar->Position;
if (Dx_SoundBuffer1->SetPan( Pan )) PanLabel->Caption = "Pan: "+IntToStr((int) Pan);
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::VolumeScrollBarChange(TObject *Sender)
{
Volume = VolumeScrollBar->Position;
if (Dx_SoundBuffer1->SetVolume( Volume )) VolumeLabel->Caption = "Volume: "+IntToStr((int) Volume);
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::FrequencyScrollBarChange(TObject *Sender)
{
// There seems to be a bug in the TScrollBar, in which the TScrollBar does
// not seem to handle ranges 0..100,000 correctly and truncates values.
// So, we've fiddled with the FrequencyScrollBar's Position, Min, Max,
// SmallChange & LargeChange properties so they fit a smaller scale 0..10,000,
// which works appropriately.
Frequency = FrequencyScrollBar->Position * 10;
if (Dx_SoundBuffer1->SetFrequency( Frequency ))	FrequencyLabel->Caption = "Frequency: "+IntToStr(Frequency);
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::Dx_SoundBuffer1Destroy(TObject *Sender)
{
Dx_SoundBuffer1->Stop();
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::FileListBox1DblClick(TObject *Sender)
{
PlayButtonClick(this);
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::StopButtonClick(TObject *Sender)
{
Dx_SoundBuffer1->Stop();
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::RewindButtonClick(TObject *Sender)
{
Dx_SoundBuffer1->SetCurrentPosition(CurrentPlayPos-DSBufferDesc1->BufferBytes/10);
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::ForwardButtonClick(TObject *Sender)
{
Dx_SoundBuffer1->SetCurrentPosition(CurrentPlayPos+DSBufferDesc1->BufferBytes/10);
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::PauseButtonClick(TObject *Sender)
{
if (Dx_SoundBuffer1->Playing)
	{
    Paused = Dx_SoundBuffer1->Stop();
    PlayingLabel->Caption = "Buffer is Paused..";
    }
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::QuitButtonClick(TObject *Sender)
{
Close();
}
//---------------------------------------------------------------------------

